/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017-2020 Yegor Bugayenko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.cactoos.iterable;

import org.cactoos.Scalar;
import org.cactoos.scalar.Unchecked;

/**
 * Repeat an element.
 *
 * <p>If you need to repeat endlessly, use {@link Endless}.</p>
 *
 * @param <T> Element type
 * @since 0.1
 */
public final class Repeated<T> extends IterableEnvelope<T> {

    /**
     * Ctor.
     * @param total The total number of repetitions
     * @param elm The element to repeat
     */
    public Repeated(final int total, final T elm) {
        this(total, () -> elm);
    }

    /**
     * Ctor.
     * @param total The total number of repetitions
     * @param elm The element to repeat
     */
    public Repeated(final int total, final Scalar<T> elm) {
        this(total, new Unchecked<T>(elm));
    }

    /**
     * Ctor.
     * @param total The total number of repetitions
     * @param item The element to repeat
     */
    public Repeated(final int total, final Unchecked<T> item) {
        super(
            new IterableOf<>(
                () -> new org.cactoos.iterator.Repeated<>(total, item)
            )
        );
    }

}
